#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Use SQLAlchemy engine to fetch a dataset from a query
"""
import os
from tempfile import NamedTemporaryFile

from jinja2.sandbox import SandboxedEnvironment
from sqlalchemy import create_engine, text


def connection():
    db_url = os.getenv("DATABASE_URL", "sqlite:///")
    db = create_engine(db_url)
    conn = db.connect()
    return conn


def process_template(obj, **kwargs):
    """Render query body using jinja2 sandbox
    TODO: Prevent variable expansion
    """
    env = SandboxedEnvironment()
    template = env.from_string(obj)
    return template.render(kwargs)


def result(sql, jinja=None, **kwargs):
    try:
        with connection() as conn:
            if jinja:
                sql = process_template(sql, **kwargs)
            cur = conn.execute(text(sql), **kwargs)
            cols = cur.keys()
            result = cur.fetchall()
            rows = [dict(zip(cols, row)) for row in result]
    except Exception as e:
        rows = [{"error": repr(e)}]
    return rows


def result_from_file(path, jinja=None, **kwargs):
    # If path doesn't exist
    if not os.path.exists(path):
        rows = [{"error": f"File '{path}' not found!"}]
        return rows

    # If it's a directory
    if os.path.isdir(path):
        rows = [{"error": f"'{path}' is a directory!"}]
        return rows

    # Read the given .sql file into memory.
    with open(path) as f:
        sql = f.read()
        rows = result(sql=sql, jinja=jinja, **kwargs)
        return rows


if __name__ == "__main__":
    os.environ["DATABASE_URL"] = "sqlite:///"
    with NamedTemporaryFile(mode="w") as temp:
        temp.write("select 'foo' as bar")
        temp.seek(0)
        file_result = result_from_file(temp.name)
    assert file_result == [{"bar": "foo"}]
    print(file_result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = "sqla-raw"
DESCRIPTION = "A tiny library for submitting raw SQL via SQLAlchemy"
URL = "https://github.com/tym-xqo/sqla-raw"
EMAIL = "thomas@yager-madden.com"
AUTHOR = "Thomas Yager-Madden"
REQUIRES_PYTHON = ">=3.6.0"
VERSION = "0.2.0"

REQUIRED = [
    "jinja2",
    "sqlalchemy",
]

EXTRAS = {"mysql": ["PyMySQL"], "pg": ["psycopg2"]}

# The rest you shouldn't have to touch too much
# ----------------------------------------------

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
try:
    with io.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
        long_description = "\n" + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = "raw"
    with open(os.path.join(here, project_slug, "__version__.py")) as f:
        exec(f.read(), about)
else:
    about["__version__"] = VERSION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(here, "dist"))
        except OSError:
            pass

        self.status("Building Source and Wheel (universal) distribution…")
        os.system("{0} setup.py sdist bdist_wheel --universal".format(sys.executable))

        self.status("Uploading the package to PyPI via Twine…")
        os.system("twine upload dist/*")

        self.status("Pushing git tags…")
        os.system("git tag v{0}".format(about["__version__"]))
        os.system("git push --tags")

        sys.exit()


# Apply above config values to actual setup
setup(
    name=NAME,
    version=about["__version__"],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=("tests",)),
    test_suite="tests",
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    setup_requires=["pytest-runner"],
    tests_require=["pytest"],
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Environment :: Web Environment",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Database :: Front-Ends",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
    ],
    # $ setup.py publish support.
    cmdclass={"upload": UploadCommand},
)
